@@ -5,16 +5,18 @@ use warnings;
 use FFI::Platypus;
 
 # ABSTRACT: An FFI function object
-our $VERSION = '0.99_06'; # TRIAL VERSION
+our $VERSION = '0.78'; # VERSION
 
 
 use overload '&{}' => sub {
   my $ffi = shift;
   sub { $ffi->call(@_) };
-}, 'bool' => sub {
+};
+
+use overload 'bool' => sub {
   my $ffi = shift;
   return $ffi;
-}, fallback => 1;
+};
 
 package FFI::Platypus::Function::Function;
 
@@ -38,20 +40,6 @@ sub attach
   $self;
 }
 
-sub sub_ref
-{
-  my($self) = @_;
-
-  my $frame = -1;
-  my($caller, $filename, $line);
-
-  do {
-    ($caller, $filename, $line) = caller(++$frame);
-  } while( $caller =~ /^FFI::Platypus(|::Function|::Function::Wrapper|::Declare)$/ );
-
-  $self->_sub_ref("$filename:$line");
-}
-
 package FFI::Platypus::Function::Wrapper;
 
 use base qw( FFI::Platypus::Function );
@@ -65,10 +53,11 @@ sub new
 sub call
 {
   my($function, $wrapper) = @{ shift() };
-  @_ = ($function, @_);
-  goto &$wrapper;
+  $wrapper->($function, @_);
 }
 
+my $counter = 0;
+
 sub attach
 {
   my($self, $perl_name, $proto) = @_;
@@ -82,36 +71,18 @@ sub attach
     $perl_name = join '::', $caller, $perl_name
   }
 
-  my $xsub = $function->sub_ref;
+  my $attach_name = "FFI::Platypus::Inner::xsub@{[ $counter++ ]}";
+  $function->attach($attach_name);
+  my $xsub = \&{$attach_name};
 
   {
-    my $code = sub {
-      unshift @_, $xsub;
-      goto &$wrapper;
-    };
-    if(defined $proto)
-    {
-      _set_prototype($proto, $code);
-    }
     no strict 'refs';
-    *{$perl_name} = $code;
+    *{$perl_name} = sub { $wrapper->($xsub, @_) };
   }
 
   $self;
 }
 
-sub sub_ref
-{
-  my($self) = @_;
-  my($function, $wrapper) = @{ $self };
-  my $xsub = $function->sub_ref;
-
-  return sub {
-    unshift @_, $xsub;
-    goto &$wrapper;
-  };
-}
-
 1;
 
 __END__
@@ -126,14 +97,14 @@ FFI::Platypus::Function - An FFI function object
 
 =head1 VERSION
 
-version 0.99_06
+version 0.78
 
 =head1 SYNOPSIS
 
  use FFI::Platypus;
  
  # call directly
- my $ffi = FFI::Platypus->new( api => 1 );
+ my $ffi = FFI::Platypus->new;
  my $f = $ffi->function(puts => ['string'] => 'int');
  $f->call("hello there");
  
@@ -148,29 +119,13 @@ context and better examples see L<FFI::Platypus>.
 
 =head1 METHODS
 
-=head2 attach
-
- $f->attach($name);
- $f->attach($name, $prototype);
-
-Attaches the function as an xsub (similar to calling attach directly
-from an L<FFI::Platypus> instance).  You may optionally include a
-prototype.
-
 =head2 call
 
  my $ret = $f->call(@arguments);
  my $ret = $f->(@arguments);
 
-Calls the function and returns the result. You can also use the
-function object B<like> a code reference.
-
-=head2 sub_ref
-
- my $code = $f->sub_ref;
-
-Returns an anonymous code reference.  This will usually be faster
-than using the C<call> method above.
+Calls the function and returns the result.  You can also use the
+function object like a code reference.
 
 =head1 AUTHOR
 
@@ -198,14 +153,6 @@ Damyan Ivanov
 
 Ilya Pavlov (Ilya33)
 
-Petr Pisar (ppisar)
-
-Mohammad S Anwar (MANWAR)
-
-Håkon Hægland (hakonhagland, HAKONH)
-
-Meredith (merrilymeredith, MHOWARD)
-
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2015,2016,2017,2018,2019 by Graham Ollis.
